@@ -369,14 +369,6 @@ class FlaskSimpleAuth:
                 return True
         return False
 
-    def _auth_first(self):
-        """Current priority authentication scheme for WWW-Authenticate."""
-        for a in self._local.auth:
-            if a in ("token", "oauth") and self._carrier == "bearer" or \
-               a in ("http-token", "basic", "http-basic", "digest", "http-digest", "password"):
-                return a
-        return None
-
     #
     # HOOKS
     #
@@ -466,17 +458,18 @@ class FlaskSimpleAuth:
     def _set_www_authenticate(self, res: Response):
         """Set WWW-Authenticate response header depending on current scheme."""
         if res.status_code == 401:
-            auth = self._auth_first()
-            if not auth:
-                pass
-            elif auth in ("token", "oauth"):  # prioritize tokens
-                res.headers["WWW-Authenticate"] = f'{self._name} realm="{self._realm}"'
-            elif auth in ("basic", "password"):
-                res.headers["WWW-Authenticate"] = f'Basic realm="{self._realm}"'
-            else:
-                assert self._http_auth
-                res.headers["WWW-Authenticate"] = self._http_auth.authenticate_header()
-            # else: scheme does not rely on WWW-Authenticate…
+            schemes = set()
+            for a in self._local.auth:
+                if a in ("token", "oauth") and self._carrier == "bearer":
+                    schemes.add(f'{self._name} realm="{self._realm}"')
+                elif a in ("basic", "password"):
+                    schemes.add(f'Basic realm="{self._realm}"')
+                elif a in ("http-token", "http-basic", "digest", "http-digest"):
+                    assert self._http_auth
+                    schemes.add(self._http_auth.authenticate_header())
+                # else: scheme does not rely on WWW-Authenticate…
+            if schemes:
+                res.headers["WWW-Authenticate"] = ", ".join(sorted(schemes))
         # else: no need for WWW-Authenticate
         return res
 
@@ -488,8 +481,10 @@ class FlaskSimpleAuth:
                 res.headers[name] = val
         return res
 
-    def _add_delay_header(self, res: Response):
-        """Add delay showing elapsed time from module perspective."""
+    def _add_fsa_headers(self, res: Response):
+        """Add convenient FSA-related headers."""
+        res.headers["FSA-Request"] = f"{request.method} {request.path}"
+        res.headers["FSA-User"] = self.current_user()
         delay = dt.datetime.timestamp(dt.datetime.now()) - self._local.start
         res.headers["FSA-Delay"] = f"{delay:.6f}"
         return res
@@ -917,7 +912,7 @@ class FlaskSimpleAuth:
         self._before_requests = conf.get("FSA_BEFORE_REQUEST", [])
         # internal hooks
         if self._mode >= Mode.DEV:
-            self._app.after_request(self._add_delay_header)
+            self._app.after_request(self._add_fsa_headers)
         self._after_requests = conf.get("FSA_AFTER_REQUEST", [])
         if self._after_requests:
             self._app.after_request(self._run_after_requests)
